unit RmvApp;

{
  Add/Remove Application for Delphi 32
  Copyright (C) 2001  Paul TOTH <tothpaul@free.fr>
  http://tothpaul.free.fr

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

}


interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

type
  TAddRemoveApp= class(TComponent)
  private
   fAppName:string;
   fDisplayName:string;
   fUninstallString:string;
   fConfirm:boolean;
   fConfirmation:string;
   EOnInstall:TNotifyEvent;
   EOnRemove:TNotifyEvent;
  protected
  public
   Constructor Create(AOwner:TComponent); override;
   procedure Install;          // add registry information for this application
   function Installed:boolean; // TRUE if this application is registered
   procedure Uninstall;        // remove registry information for this application
   function WriteReg(Key,Value:string):boolean;
   function ReadReg(Key:string):string;
   function ProcessMacro(s:string):string;
  published
   property AppName:string read fAppName write fAppName; // Registry KeyName for this application
   property DisplayName:string read fDisplayName write fDisplayName; // Display label (in Add/Remove application)
   property UninstallString:string read fUninstallString write fUninstallString; // parameter for remove
   property Confirm:boolean read fConfirm write fConfirm default true; // should we confirm uninstallation ?
   property Confirmation:string read fConfirmation write fConfirmation; // confirmation message
   property OnInstall:TNotifyEvent read EOnInstall write EOnInstall; // events...
   property OnRemove:TNotifyEvent read EOnRemove write EOnRemove;
  end;

procedure Register;

implementation

const
 WINKEY='Software\Microsoft\Windows\CurrentVersion\Uninstall\';

Constructor TAddRemoveApp.Create(AOwner:TComponent);
 begin
  inherited;
  fAppName:='MySoft.App1';
  fDisplayName:='MySoft Installer for ...';
  fUninstallString:='$EXENAME /remove';
  fConfirm:=True;
  fConfirmation:='Voulez-vous supprimer cette application ?';
 end;

function TAddRemoveApp.ProcessMacro(s:string):string;
 var
  i,j:integer;
  ss:string;
 begin
  i:=pos('$EXENAME',s);
  while i>0 do begin
   Delete(s,i,8);
   Insert(Application.ExeName,s,i);
   i:=pos('$EXENAME',s);
  end;
  i:=pos('$PATH(',s);
  while i>0 do begin
   j:=pos(')',copy(s,i,length(s)));
   if j=0 then break;
   ss:=copy(s,i+6,j-6);
   Delete(s,i,j);
   Insert(ExtractFilePath(ss),s,i);
   i:=pos('$PATH(',s);
  end;
  Result:=s;
 end;

function TAddRemoveApp.WriteReg(Key,Value:string):boolean;
 Var
  Handle:HKEY;
  Disposition:integer;
 begin
  Result:=RegCreateKeyEx(HKEY_LOCAL_MACHINE,PChar(WINKEY+fAppName),0,nil,REG_OPTION_NON_VOLATILE,KEY_ALL_ACCESS,nil,Handle,@Disposition)=ERROR_SUCCESS;
  if Result then begin
   Result:=RegSetValueEx(Handle,pchar(Key),0,REG_SZ,pchar(Value),Length(Value))=ERROR_SUCCESS;
   RegCloseKey(Handle);
  end;
 end;

function TAddRemoveApp.ReadReg(Key:string):string;
 Var
  Handle:HKEY;
  RegType:integer;
  DataSize:integer;
 begin
  Result:='';
  if (RegOpenKeyEx(HKEY_LOCAL_MACHINE,PChar(WINKEY+fAppName),0,KEY_ALL_ACCESS,Handle)=ERROR_SUCCESS) then begin
   if RegQueryValueEx(Handle,pchar(Key),nil,@RegType,nil,@DataSize)=ERROR_SUCCESS then begin
    SetLength(Result,DataSize);
    RegQueryValueEx(Handle,pchar(Key),nil,@RegType,PByte(pchar(Result)),@DataSize);
    SetLength(Result,DataSize-1);
   end;
   RegCloseKey(Handle);
  end;
 end;

procedure TAddRemoveApp.Install;
 begin
  WriteReg('DisplayName',fDisplayName);
  WriteReg('UninstallString',ProcessMacro(fUninstallString));
  if Assigned(EOnInstall) then EOnInstall(self);
 end;

function TAddRemoveApp.Installed:boolean;
 begin
  Result:=ReadReg('DisplayName')=fDisplayName;
 end;

procedure TAddRemoveApp.Uninstall;
 begin
  if fConfirm then begin
   If Application.MessageBox(pchar(fConfirmation),pchar(fDisplayName),MB_YESNO)=IDNO then exit;
  end;
  if Assigned(EOnRemove) then EOnRemove(self);
  RegDeleteKey(HKEY_LOCAL_MACHINE,PChar(WINKEY+fAppName));
 end;

procedure Register;
begin
  RegisterComponents('MySoft', [TAddRemoveApp]);
end;

end.
